Package: ctsGE
Title: Clustering of Time Series Gene Expression data
Version: 1.36.0
Authors@R: c(person("Michal", "Sharabi-Schwager",
                email = "michalsharabi@gmail.com", role = c("aut", "cre")),
                person("Ron", "Ophir",
                    email = "ron@volcani.agri.gov.il", role = "aut"))
Description: Methodology for supervised clustering of potentially many predictor
                variables, such as genes etc., in time series datasets
                Provides functions that help the user assigning genes to
                predefined
                set of model profiles.
Depends: R (>= 3.2)
Imports: ccaPP, ggplot2, limma, reshape2, shiny, stats, stringr, utils
Suggests: BiocStyle, dplyr, DT, GEOquery, knitr, pander, rmarkdown,
        testthat
URL: https://github.com/michalsharabi/ctsGE
BugReports: https://github.com/michalsharabi/ctsGE/issues
License: GPL-2
RoxygenNote: 5.0.1
VignetteBuilder: knitr
biocViews: ImmunoOncology, GeneExpression, Transcription,
        DifferentialExpression, GeneSetEnrichment, Genetics, Bayesian,
        Clustering, TimeCourse, Sequencing, RNASeq
git_url: https://git.bioconductor.org/packages/ctsGE
git_branch: RELEASE_3_22
git_last_commit: f0f5237
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 03:23:58 UTC; biocbuild
Author: Michal Sharabi-Schwager [aut, cre],
  Ron Ophir [aut]
Maintainer: Michal Sharabi-Schwager <michalsharabi@gmail.com>

# Generated by roxygen2: do not edit by hand

export(ClustIndexes)
export(PlotIndexesClust)
export(PreparingTheIndexes)
export(ctsGEShinyApp)
export(index)
export(readTSGE)
import(ccaPP)
import(ggplot2)
import(limma)
import(reshape2)
import(shiny)
import(stats)
import(stringr)
import(utils)

# ctsGE
Clustering of time series gene expression data
## Installing ctsGE


```{r,eval=FALSE,warning=FALSE,message=FALSE}
BiocManager::install("ctsGE")
```
**OR**

```{r,eval=FALSE,warning=FALSE,message=FALSE}
devtools::install_github("michalsharabi/ctsGE")
```

**OR:**

*Download [source](https://github.com/michalsharabi/ctsGE/tree/master/source)*
```{r, eval=FALSE,warning=FALSE,message=FALSE }
install.packages("ctsGE.tar.gz",repos=NULL,type="source")
```
## ctsGE usage
Tutotial and examples can be found  [here](https://htmlpreview.github.io/?https://github.com/michalsharabi/ctsGE/blob/master/inst/doc/ctsGE.html)

#' GUI for interactive exploration of gene expression data.
#'
#' Produce and launch Shiny app for interactive exploration of gene expression
#'  data.
#' For more information about shiny apps \cite{http://shiny.rstudio.com/}
#'
#' @param rts list of an expression data that made by readTSGE
#' @param min_cutoff A numeric the lower limit range to calculate the optimal
#' cutoff for the data, default to 0.5
#' \emph{See \code{\link{PreparingTheIndexes}}}.
#' @param max_cutoff A numeric the upper limit range to calculate the optimal
#' cutoff for the data, default to 0.7
#' \emph{See \code{\link{PreparingTheIndexes}}}.
#' @param mad.scale A boolean defaulting to TRUE as to what method of scaling
#' to use.
#'  Default median-base scaling. FALSE, mean-base scaling
#' @param title Character, the title at the header panel. default to NULL.
#'
#' @return Creates a shiny application and opens a shinyapp.io web page
#'
#' @details The `ctsGEShinyApp` function takes the ctsGE object
#' and opens an html page as a GUI. On the web page, the user
#' chooses the profile to visualize and the number of clusters (k parameter for
#'  K-means) to show. The line graph of the profile separated into the clusters
#'  will show in the main panel, and a list of the genes and their expressions
#'  will also be available. The tables and figures can be downloaded.
#'
#' @seealso shiny::ShinyApp
#' @examples
#'
#' \dontrun{
#' data_dir <- system.file("extdata", package = "ctsGE")
#' files <- dir(path=data_dir,pattern = "\\.xls$")
#' rts <- readTSGE(files, path = data_dir,
#' labels = c("0h","6h","12h","24h","48h","72h") )
#' ctsGEShinyApp(rts)}
#'
#'
#' @export
#' @import  stats shiny
#'

ctsGEShinyApp <- function(rts,min_cutoff=0.5,max_cutoff=0.7,
                          mad.scale=TRUE,title=NULL) {
    prts <- PreparingTheIndexes(rts,min_cutoff,max_cutoff,mad.scale)
    idx <- as.character(unique(prts$index[,"index"]))

    clusters <- function(tbl,g){
        #set.seed(100)
        tmp <- as.matrix(tbl[,c(1:rts$timePoints)])
        fit <- kmeans(tmp, g,nstart = 25)
        kmeans.groups <-
            cbind(merge(data.frame(fit$cluster),tmp,by="row.names",all=TRUE),
                  index=tbl$index)
        colnames(kmeans.groups)[1:2] <- c("genes","clusters")

        return(kmeans.groups)
    }

    get_plot_output_list <- function(plot_list) {
        #set.seed(100)

        # Insert plot output objects the list
        plot_output_list <- lapply(1:length(plot_list), function(i) {
            plotname <- names(plot_list)[i]
            plot_output_object <-
                shiny::plotOutput(plotname, height = 280, width = 250)
            plot_output_object <- shiny::renderPlot({
                gg <- plot_list[[i]]
                print(gg)
            })
        })
        return(do.call(shiny::tagList,plot_output_list))
    }

    shiny::shinyApp(

        ui = shiny::pageWithSidebar(
            shiny::headerPanel(title),
            shiny::sidebarPanel(width = 2,
                                shiny::selectInput("index","Select an Index:",
                                                   choices = idx,
                                                   selected = idx[1]),

                                shiny::sliderInput("n", "Number of clusters",
                                                   min = 1,max= 10,
                                                   value= 1,step= 1),
                                shiny::checkboxInput("scale",
                                                     "Unscaled values",value = FALSE)
            ),
            shiny::mainPanel(width = 10,
                             shiny::tabsetPanel(
                                 shiny::tabPanel("Time series",
                                                 icon =shiny::icon("line-chart"),
                                                 shiny::uiOutput("plots")),
                                 shiny::tabPanel( "Genes Table",
                                                  icon = shiny::icon("table"),
                                                  shiny::uiOutput("table"))
                             )
            )
        ),#pageWithSidebar

        server = function(input, output,session) {
            # filter input$index
            filtered <- shiny::reactive({
                if (is.null(input$index)) {
                    return(NULL)
                }
                set.seed(100)
                if(!input$scale){
                    PlotIndexesClust(prts,input$index,k = input$n)
                }else{
                    PlotIndexesClust(prts,input$index,k = input$n,
                                     scaling = FALSE)}
            })

            shiny::observe({
                list_plot <- filtered()[[2]]
                output$plots <- shiny::renderUI({

                    get_plot_output_list(list_plot)
                })
                tbl <- filtered()[[1]]
                if(ncol(tbl) > 7){
                tbl <- cbind(genes=rownames(tbl),
                             desc =as.factor(tbl$desc),
                             clusters=as.factor(tbl$clusters),
                             data.frame(tbl[,prts$samples]),
                             index=input$index)
                } else {
                    tbl <- cbind(genes=rownames(tbl),
                                 clusters=as.factor(tbl$clusters),
                                 data.frame(tbl[,prts$samples]),
                                 index=input$index)}

                rownames(tbl) <- NULL
                output$table <- shiny::renderUI({
                    if (is.null(tbl)) {return()}

                    output$tmp <-
                        DT::renderDataTable(tbl,
                                            rownames = FALSE,
                                            extensions ='Buttons',
                                            options = list(dom =
                                                               'TB<"clear">lfrtip',
                                                           buttons = c('copy',
                                                                       'csv',
                                                                       'excel',
                                                                       'print'),
                                                           lengthMenu =
                                                               list(c(15,50,100,
                                                                      nrow(tbl)),
                                                                    c('15','50',
                                                                      '100','All')
                                                               ))

                        )
                    DT::dataTableOutput("tmp")
                })#renderUI
            })#end observe
        }
    )
}

## ----echo = FALSE, message = FALSE--------------------------------------------
library(ctsGE)
library(pander)
library(rmarkdown)

## ----eval=FALSE,warning=FALSE,message=FALSE-----------------------------------
# if (!requireNamespace("BiocManager", quietly=TRUE))
#     install.packages("BiocManager")
# BiocManager::install("ctsGE")

## ----eval=FALSE,message=FALSE, warning=FALSE----------------------------------
# library(GEOquery)
# 
# gse2077 <- getGEO('GSE2077')
# gseAssays <- Biobase::assayData(gse2077)
# gseExprs <- Biobase::assayDataElement(gseAssays[[1]][,c(1:6)],'exprs')
# 
# # list of the time series tables use only 6 samples
# gseList <- lapply(1:6,function(x){data.frame(Genes = rownames(gseExprs),Value = gseExprs[,x])})
# names(gseList) <- colnames(gseExprs)

## ----eval=FALSE,message=FALSE,warning=FALSE-----------------------------------
# rts <- readTSGE(gseList,labels = c("0h","6h","12h","24h","48h","72h"))

## ----message=FALSE,warning=FALSE----------------------------------------------
data_dir <- system.file("extdata", package = "ctsGE")
files <- dir(path=data_dir,pattern = "\\.xls$")

## ----message=FALSE,warning=FALSE----------------------------------------------

rts <- readTSGE(files, path = data_dir, labels = c("0h","6h","12h","24h","48h","72h") )

## ----message=FALSE,warning=FALSE----------------------------------------------
names(rts)
rts$timePoints
head(rts$samples)
head(rts$tags)

## ----echo=FALSE,results='asis'------------------------------------------------

panderOptions("table.style","rmarkdown")
pander(head(rts$tsTable))


## ----message=FALSE,warning=FALSE----------------------------------------------
prts <- PreparingTheIndexes(x = rts, min_cutoff=0.5, max_cutoff=0.7, mad.scale = TRUE)

## ----message=FALSE,warning=FALSE----------------------------------------------
prts$cutoff

## ----message=FALSE,warning=FALSE,echo=FALSE-----------------------------------
library(dplyr)

count_zero <- 
  function(x){
    sum(strsplit(x,"")[[1]]==0)}


tbl <- prts$index %>%
    # counting genes at each index
    group_by(index)%>% summarise(size=length(index)) %>% 
    # counting the number of zeros at each index
    group_by(index)%>% mutate(nzero=count_zero(as.character(index))) %>% 
    # groups genes by the number of zeros and sum them
    group_by(nzero) %>% summarise(genes=round(sum(size)/12625,1)) 

tmp = which(0:6%in%tbl$nzero==0)-1
tmp_df = data.frame(nzero=tmp,genes=rep(0,length(tmp)))
tbl <- bind_rows(tbl,tmp_df) %>% arrange(nzero)
labs <- seq(0,max(tbl$genes), by = 0.2)
barplot(tbl$genes, 
        main = paste("Number of zeros in indexes with cutoff =",prts$cutoff), 
        names.arg = tbl$nzero,axes = FALSE, xlab="Number of Zeros")
axis(side = 2, at = labs, labels = paste0(labs * 100, "%"))

## ----message=FALSE,warning=FALSE----------------------------------------------
prts <- PreparingTheIndexes(x = rts, mad.scale = TRUE) 
names(prts)

## ----message=FALSE,echo=FALSE,warning=FALSE,results="asis"--------------------

panderOptions("table.style","simple")
pander(head(prts$scaled)) 

## ----message=FALSE,echo=FALSE,warning=FALSE,results="asis"--------------------

panderOptions("table.style","simple")
pander(head(prts$index)) 

## ----message=FALSE, warning=FALSE,eval=FALSE----------------------------------
# ClustIndexes <- ClustIndexes(prts, scaling = TRUE)
# names(ClustIndexes)
# # table of the index and the recommended k that were found by the function
# head(ClustIndexes$optimalK)
# 
# # Table of clusters index for each gene
# head(ClustIndexes$ClusteredIdxTable)

## ----message=FALSE,warning=FALSE----------------------------------------------
indexPlot <- PlotIndexesClust(prts,idx = "1100-1-1",scaling = TRUE)
names(indexPlot)

## ----message=FALSE,warning=FALSE,echo=FALSE-----------------------------------
length(indexPlot$graphs)

## ----message=FALSE,warning=FALSE,results="asis",echo=FALSE--------------------

panderOptions("table.style","rmarkdown")
pander(head(indexPlot[[1]]))

## ----message=FALSE,warning=FALSE----------------------------------------------
indexPlot$graphs

## ----message=FALSE,warning=FALSE,eval=FALSE-----------------------------------
# library(shiny)
# library(DT)
# ctsGEShinyApp(rts)

#' Define an expression index for each gene
#'
#' Reads the table of genes expression and return an expression index for each
#' gene.
#'
#'
#' @param x list of an expression data that made by readTSGE
#' @param min_cutoff A numeric the lower limit range to calculate the optimal
#' cutoff for the data, default to 0.5  \emph{See Details}.
#' @param max_cutoff A numeric the upper limit range to calculate the optimal
#' cutoff for the data, default to 0.7  \emph{See Details}.
#' @param mad.scale A boolean defaulting to TRUE as to what method of
#' scaling to use.
#' Default median-base scaling. FALSE, mean-base scaling.
#' @return list object is returned as output with the relative
#' standarization table in \emph{object$scaled},
#' and the indexes table in \emph{object$index}
#'
#' @details 1. First, the expression matrix is standardized. The function
#' default standardizing method is a median-based scaling; alternatively, a
#' mean-based scaling can be used. The new scaled values represent the distance
#' of each gene at a certain time point from its center, median or mean,
#' in median absolute deviation (MAD) units or standard deviation (SD) units,
#' respectively.
#'
#' 2. The function compute the cutoff value following the idea that the
#' clustering will be performed on small gene groups, an optimal cutoff value
#' will be one that will minimize the number of genes in each group,
#' i.e., generate index groups of equal size. The chi-squared values will be
#' generate for each cutoff value (from min_cutoff to max_cutoff parameter
#' in increments of 0.05) the cutoff that generate the lowest chi-squared is
#' chosen.
#'
#' 3. Next, the standardized values are converted to index values that indicate
#' whether gene expression is above, below or within the limits around the
#' center of the time series, i.e., **1 / -1 / 0**, respectively. The cutoff
#' parameter determines the limits around the gene-expression center.
#' Then the function calculates the index value at each
#' time point according to:
#'
#' \enumerate{
#'      \item \bold{0:}  standardized value is within the limits (+/- cutoff)
#'      \item \bold{1:}  standardized value exceeds the upper limit (+ cutoff)
#'      \item \bold{-1:} standardized value exceeds the lower limit (- cutoff)}
#'
#'
#'
#' @examples
#' data_dir <- system.file("extdata", package = "ctsGE")
#' files <- dir(path=data_dir,pattern = "\\.xls$")
#' rts <- readTSGE(files, path = data_dir,
#' labels = c("0h","6h","12h","24h","48h","72h"), skip = 10625 )
#' prts <- PreparingTheIndexes(rts)
#' prts$cutoff # the optimal cutoff
#'
#'@seealso \code{\link{scale}} \code{\link{index}}
#'
#' @export
#' @import stats ccaPP stringr
#'
PreparingTheIndexes = function(x,min_cutoff=0.5,max_cutoff=0.7,mad.scale=TRUE){
    # function to test cutoff values
    TstCutoff <- function(x,min_cutoff=0.5,max_cutoff=0.7){
        c <- seq(min_cutoff,max_cutoff,0.05)
        idx_tbl <- lapply(c,function(i){apply(x,1,index,cutoff=i)})
        idx_str <- lapply(idx_tbl,
                          function(y){apply(t(y),1,stringr::str_c,collapse="")})

        chisq_val <- lapply(idx_str,
                            function(z){chisq.test(table(z))$statistic[[1]]})
        i = which(unlist(chisq_val)==min(unlist(chisq_val)))
        return(list(cutoff=c[i],idx_tbl=t(idx_tbl[[i]]),idx_str = idx_str[[i]]))}

    if(!is.list(x)) stop ( "data must be a list")
    if(sum(names(x)%in%c("tsTable","samples","tags","timePoints","desc" )) < 4)
        stop ( "Your List miss one or more of theses objects:
               tsTable, samples, tags, timePoints")
    if(min_cutoff > max_cutoff)
        stop ("min_cutoff value must be smaller than max_cutoff")
    tp <- x$timePoints
    # compute the MAD faster and return median and MAD for each row
    fastMad <- apply(t(x$tsTable),2,ccaPP::fastMAD)
    mad_val <- grep("MAD",names(unlist(fastMad)))
    med_val <- grep("center",names(unlist(fastMad)))

    x$scaled <-
        tmp <-
        t(scale(t(x$tsTable),unlist(fastMad)[med_val],unlist(fastMad)[mad_val]))

    if(!mad.scale) x$scaled <- tmp <-  t(scale(t(x$tsTable)))


    tmp <- TstCutoff(tmp,min_cutoff,max_cutoff)
    x$index <- cbind(as.data.frame(tmp$idx_tbl),index=tmp$idx_str)
    x$cutoff <- tmp$cutoff

    return(x)
}



#' Indexing function
#'
#' Takes a numeric vector and return an expression index
#' (i.e., a sequence of 1,-1, and 0)
#' @param x A numeric
#' @param cutoff A numeric, dermine the threshold for indexing
#' @return Gene expression index
#' @details The function defines limits around the center (median or mean),
#' +/- cutoff value in median absolute deviation (MAD) or standard deviation
#' (SD) units respectively.The user defines a parameter cutoff that determines
#' the limits around the gene-expression center. Then the function calculates
#' the index value at each time point according to:
#'
#' \enumerate{
#'      \item \bold{0:}  standardized value is within the limits (+/- cutoff)
#'      \item \bold{1:}  standardized value exceeds the upper limit (+ cutoff)
#'      \item \bold{-1:} standardized value exceeds the lower limit (- cutoff)}
#'
#' @seealso \code{\link{PreparingTheIndexes}}
#' @examples
#' rawCounts <-
#' c(103.5, 75.1, 97.3, 27.12, 34.83, 35.53, 40.59, 30.84, 16.39, 29.29)
#'
#' (sCounts <- scale(rawCounts)[,1])# standardized mean-base scaling
#'
#' cutoff <- seq(0.2,2,0.1)    # different cutoff produce different indexes
#'
#' for(i in cutoff){print(index(sCounts,i))}
#' @export
#'

index=function(x,cutoff){
    y=x
    y[x>=cutoff]=1
    y[x<=cutoff]=-1
    y[x>-cutoff&x<cutoff]=0
    return(y)}

data_dir <- system.file("extdata", package = "ctsGE")
files <- dir(path=data_dir,pattern = "\\.xls$")
rts <- readTSGE(files, path = data_dir, skip = 10625 )
prts <- PreparingTheIndexes(rts)
tsCI <- ClustIndexes(prts)

expect_is(rts,"list")
expect_is(prts,"list")
expect_is(tsCI,"list")
expect_equal(length(rts),4)||expect_equal(length(rts),5)
expect_equal(length(prts),7)||expect_equal(length(prts),8)
expect_equal(sum(is.na(tsCI$optimalK[,2])),0)
expect_error(PlotIndexesClust(prts,idx="lkj"))
expect_error(PlotIndexesClust(prts,idx=000000))
expect_error(PlotIndexesClust(rts,idx="000000"),
             "Please run PreparingTheIndexes first")

#' Graphic visualization of an index
#'
#' The function generates graphs and tables of a specific index and its
#' clusters. The user decides whether to supply the k or let the function
#' calculate the k for the selected index
#'
#' @param x list of expression data and their indexes after running
#' \code{\link{PreparingTheIndexes}}
#' @param idx A character, the index to plot
#' (e.g., for 8 time points "11100-1-1-1")
#' @param k A numeric, number of clusters. If not given the function will
#' calculate what is the optimal k for the index.
#' @param scaling A boolean, default to TRUE,
#' does the data should be standardized before clustered with K-means.
#'
#' @return A list with two objects:
#'  \enumerate{\item Table of of a specific index and its clusters
#'             \item Gene expression pattern graphs
#'             for each one of the clusters}
#'
#' @seealso \code{\link{ggplot}},  \code{\link{kmeans}},
#'  \code{\link{ClustIndexes}}
#'
#' @examples
#'
#' data_dir <- system.file("extdata", package = "ctsGE")
#' files <- dir(path=data_dir,pattern = "\\.xls$")
#' rts <- readTSGE(files, path = data_dir,
#' labels = c("0h","6h","12h","24h","48h","72h"), skip = 10625 )
#' prts <- PreparingTheIndexes(rts)
#' pp <- PlotIndexesClust(prts,idx="00101-1")
#' pp$graphs # plots the line graphs
#'
#' @export
#' @import ggplot2 reshape2 stats
#'
#'

PlotIndexesClust = function(x,idx,k=NULL,scaling=TRUE){
    set.seed(100)
    plot <- list()
    ggplot_list <- list()
    clust_tbl <-  list()
    kindex <-  list()
    genes <- rownames(x$index[x$index[,"index"]==idx,])

    if(length(x) < 7){stop("Please run PreparingTheIndexes first")}
    if(!scaling){tbl <- x$tsTable[genes,]
    } else{tbl <- x$scaled[genes,]}

    if(length(genes) > 1 ) {tbl <- as.data.frame(tbl)
    }else{
        tbl <- data.frame(t(matrix(tbl)))
        colnames(tbl) <- x$samples
        rownames(tbl) <- genes
    }
    if(!is.null(k)){
        fit_km <- kmeans(tbl,k)
        tbl[names(fit_km$cluster),"clusters"] <- fit_km$cluster
        K <- k
    }else{
        # checks if indexes were already made with PreparingTheIndexes
        if("optimalK" %in% names(x)){
            k <- x$optimalK[rownames(x$optimalK)==idx,"k"]
            tbl1 <- x$ClusteredIdxTable
            tags <- rownames(tbl1[tbl1[,"index"]==idx,])
            tbl[tags,"clusters"] <- tbl1[tbl1[,"index"]==idx,"clusters"]
        }else{
            if(length(genes) > 9){
                tbl <- as.data.frame(tbl)
                k = 1
                fit_km <- kmeans(tbl,k,nstart = 25)
                opt <- fit_km$tot.withinss/fit_km$totss < 0.2
                while (!opt) {
                    k=k+1
                    # Apply k-means to tbl: fit_km
                    fit_km <- kmeans(tbl,k,nstart = 25)
                    opt <- fit_km$tot.withinss/fit_km$totss < 0.2
                }
                K <-  k #where: WSS / TSS < 0.2 this is the optimal k
                clust <-   fit_km$cluster
            }else{
                K <-  1
                clust <- rep(1,nrow(tbl))}
            kindex[[idx]] <-  c(nrow(tbl),K)
            clust_tbl[[idx]] <- data.frame(clusters=clust,index=idx)
            rownames(clust_tbl[[idx]]) <- rownames(tbl)
            tbl[,"clusters"] <- clust
        }
    }

    tmp <- cbind(tags=genes,tbl)
    for(i in 1:K){
        x.m <- reshape2::melt(tmp[tmp$cluster==i,c("tags",x$samples)])
        colnames(x.m) <- c("tags","tp","exp")
        gplot <-
            ggplot2::ggplot(x.m,
                            ggplot2::aes_string(y="exp",x="tp",colour="tags",
                                                group="tags"))+
            ggplot2::labs(title =paste0("Index: ",idx," Cluster: ",i),
                          x = "Time", fill= NULL, y = "Expression Value") +
            ggplot2::theme(legend.position="none")

        gplot <- gplot +ggplot2::geom_line()
        name <- paste0("Clust_",i,"_",idx)
        ggplot_list[[name]] <- gplot
    }
    if ("desc" %in% names(x)) {tbl <- cbind(desc=x$desc[genes,], tbl)}
    plot[[name]] <- tbl
    plot$graphs <- ggplot_list
    structure(plot,class = "list")
}

#' Read and merge a set of files containing gene expression data
#'
#' Reads and merges a set of text files containing normalized gene expression
#' data
#'
#' @param files character vector of filenames,
#' or alternative a named list of tables for each time point.
#' @param path character string giving the directory containing the files.
#' The default is the current working directory.
#' @param columns numeric vector stating which two columns contain the tag
#' names and counts, respectively
#' @param labels character vector giving short names to associate with the
#' libraries.
#' @param desc character vector with genes description (annotation),default to
#' NULL
#' Defaults to the file names.
#' @param ... other are passed to read.delim
#'
#' @return A list with four objects:
#'  \enumerate{\item expression matrix
#'             \item samples names
#'             \item tags - genes name
#'             \item timePoints - number of time points}
#'
#' @details As input, the ctsGE package expects normalized expression table,
#' where rows are genes and columns are samples
#' Each file is assumed to contained digital gene expression data for one
#' sample (or library), with transcript or gene identifiers in the first
#' column and expression values in the second column.
#' Transcript identifiers are assumed to be unique and not repeated in any one
#' file.
#' By default, the files are assumed to be tab-delimited and to contain column
#'    headings.
#' The function forms the union of all transcripts and creates one big table
#' with zeros where necessary.
#' When reading the normalized expression values the function check whether
#' there are rows that their median absolute deviation (MAD) value equal to zero
#' and remove these rows. This step is important in order to continue to the
#' next step of indexing the data.
#' The function will output a message of how many genes were remove.
#'
#' @examples
#' ## Read all .txt files from current working directory
#' data_dir <- system.file("extdata", package = "ctsGE")
#' files <- dir(path=data_dir,pattern = "\\.xls$")
#'
#' # reading only 2000 genes
#' rts <- readTSGE(files, path = data_dir,
#'  labels = c("0h","6h","12h","24h","48h","72h"), skip = 10625 )
#' @export
#' @import limma stats utils
#'
#'
readTSGE = function(files,path = NULL,columns=c(1,2),labels = NULL,desc = NULL,
                    ...){
    x <- list()
    if(!is.list(files)){
        d <- taglist <- list()
        for (fn in files) {
            if (!is.null(path))
                fn <- file.path(path, fn)
            d[[fn]] <- read.delim(fn, ..., stringsAsFactors = FALSE)
            taglist[[fn]] <- as.character(d[[fn]][, columns[1]])
            if (any(duplicated(taglist[[fn]]))) {
                stop(paste("Repeated tag sequences in", fn))
            }
        }
    }

    else{
        d <- files
        taglist <- list()
        for (fn in 1:length(files)) {
            taglist[[fn]] <- as.character(d[[fn]][,columns[1]])
            if (any(duplicated(taglist[[fn]]))) {
                stop(paste(
                    "Repeated tag sequences in table no.",i,"in the list"))
            }
        }
    }

    tags <- unique(unlist(taglist))
    ntags <- length(tags)
    nfiles <- length(files)
    x$tsTable <- matrix(0, ntags, nfiles)
    rownames(x$tsTable) <- tags
    colnames(x$tsTable) <- labels

    if (is.null(colnames(x$tsTable))){
        if(!is.list(files)){ colnames(x$tsTable) <-
            x$samples <- limma::removeExt(files)
        }else{colnames(x$tsTable) <- x$samples <- names(files)}
    }else{x$samples <- colnames(x$tsTable)}
    for (i in 1:nfiles) {
        aa <- match(taglist[[i]], tags)
        x$tsTable[aa, i] <- as.numeric(d[[i]][, columns[2]])
    }

    x$tags <- tags
    x$timePoints <- nfiles

    ## filtering out genes with low expression
    tmp <- apply(t(x$tsTable),2,mad)
    if(sum(tmp==0)){
        x$tsTable <- x$tsTable[names(which(tmp!=0)),]
        x$tags <- names(which(tmp!=0))
        print(paste0(sum(tmp==0)," Genes were remove"))}

    if(!is.null(desc)){
        x$desc <-  desc
        if(sum(tmp==0)){
            desc <- desc[which(tmp!=0)]}
        x$desc <- as.data.frame(desc)
        rownames(x$desc) <- x$tags
        colnames(x$desc) <- "desc"}




    structure(x,class = "list")
}

#' Clustering the indexes applying K-means
#'
#' Clustering each index, that was predifined by
#' \code{\link{PreparingTheIndexes}}, with \code{\link{kmeans}}.
#'
#' @param x  list of expression data and their indexes after running
#' \code{\link{PreparingTheIndexes}}
#' @param scaling Boolean parameter, does the data should be standardized
#' before clustered. Default = TRUE
#'
#'
#' @return  list object is returned as output,
#' with the relative culstered indexes table in
#'  \emph{object$ClusteredIdxTable},
#' and the number of clusters for each index in \emph{object$optimalK}
#'
#'
#' @details The clustering is done with K-means. To choose an optimal k for
#'  K-means clustering, the Elbow method was applied, this method looks at the
#'  percentage of variance explained as a function of the number of clusters: the
#'  chosen number of clusters should be such that adding another cluster does
#'  not give much better modeling of the data. First, the ratio of the
#'  within-cluster sum of squares (WSS) to the total sum of squares (TSS) is
#'  computed for different values of k (i.e., 1, 2, 3 ...). The WSS, also known
#'  as sum of squared error (SSE), decreases as k gets larger. The Elbow method
#'  chooses the k at which the SSE decreases abruptly. This happens when the
#'  computed value of the WSS-to-TSS ratio first drops from 0.2.
#'
#'  Running \code{\link{kmeans}} and calculating the optimal k for each one of
#'  the indexes in the data could take a long time. To shorten the procedure the
#'  user can skip this step altogether and directly view a specific index and
#'  its clusters by running either the \code{\link{PlotIndexesClust}} or the
#'  \code{\link{ctsGEShinyApp}} function.
#'
#'  By default data is standardize before clustering,for clustering
#'  the raw counts set the \command{\strong{scaling}} parameter to FALSE.
#'
#'
#' @examples
#'
#' data_dir <- system.file("extdata", package = "ctsGE")
#' files <- dir(path=data_dir,pattern = "\\.xls$")
#' rts <- readTSGE(files, path = data_dir,
#' labels = c("0h","6h","12h","24h","48h","72h"), skip = 10625 )
#' prts <- PreparingTheIndexes(rts)
#'
#' tsCI <- ClustIndexes(prts)
#'
#' head(tsCI$ClusteredIdxTable) #the table with the clusterd indexes
#' head(tsCI$optimalK) #the table with the number of clusters for each index
#'
#' @seealso \code{\link{kmeans}}, \code{\link{PlotIndexesClust}}
#' @export
#' @import stats
#'
ClustIndexes = function(x,scaling=TRUE){
    clust_tbl <-  list()
    kindex <-  list()

    if(scaling){tmp <-
        cbind(as.data.frame(x$scaled),index=x$index[rownames(x$scaled),
                                                    ncol(x$index)])
    }else { tmp <-
        cbind(as.data.frame(x$tsTable),index=x$index[rownames(x$scaled),
                                                     ncol(x$index)])}


    for(idx in as.character(unique(tmp$index))){
        tbl <- tmp[tmp$index==idx,1:x$timePoints]
        if(nrow(tbl) > 9){
            k = 1
            fit_km <- kmeans(tbl,k,nstart = 25)
            opt <- fit_km$tot.withinss/fit_km$totss < 0.2
            while (!opt) {
                k=k+1
                # Apply k-means to tbl: fit_km
                fit_km <- kmeans(tbl,k,nstart = 25)
                opt <- fit_km$tot.withinss/fit_km$totss < 0.2
            }
            K <-  k #where: WSS / TSS < 0.2 this is the optimal k
            clust <-   fit_km$cluster
        }else{
            K <- 1
            clust <- rep(1,nrow(tbl))}

        kindex[[idx]] <-  c(nrow(tbl),K)
        clust_tbl[[idx]] <- data.frame(clusters=clust,index=idx)
        rownames(clust_tbl[[idx]]) <- rownames(tbl)
    }

    names(clust_tbl) <-  NULL
    x$optimalK <-  do.call("rbind",kindex)
    colnames(x$optimalK) <- c("tags","k")
    x$ClusteredIdxTable <- do.call("rbind",clust_tbl)
    return(x)
}

---
title: "ctsGE Package"
author: "Michal Sharabi-Schwager, Ron Ophir"
date: "`r Sys.Date()`"
abstract: >
 A prerequisite to clustering noisy data, such as gene-expression data, 
 is the filtering step. ctsGE is an R package that performs clustering of 
 time-series gene-expression data: as an alternative to the filtering step, 
 it applies a sorting step in which all of the data are divided into small 
 groups according to how the time points are related to the time-series median. 
 Then, clustering is performed separately on each group, using a 
 "structure-based" dissimilarity concept(e.g., Lin and Li^[Lin J, Li Y (2009).
 Finding Structural Similarity in Time Series Data Using Bag-of-Patterns 
 Representation. In Proceedings of the 21st International Conference on 
 Scientific and Statistical Database Management (SSDBM 2009), Marianne Winslett
 (Ed.). Springer-Verlag, Berlin, Heidelberg, pp. 461–477. ISBN 978-3-642-02278-4.
 ]; Corduas^[Corduas M (2010). Mining Time  Series Data: A Selective Survey.
 In Proceedings of the 6th Conference of the Classification and Data Analysis 
 Group of the SocietàItaliana di Statistica, F Palumbo, CN Lauro,MJ Greenacre 
 (Eds.). Springer-Verlag, Berlin, Heidelberg, pp. 355–362. 
 ISBN 978-3-642-03739-9.]). Thus, the clustering is done in two steps.
 First an expression index  (i.e., a sequence of 1,-1, and 0) is defined that 
 divides genes into groups according to their expression profiles, and then 
 genes with the same index are clustered by applying **K-means**. The **ctsGE** 
 package also provides an interactive tool to visualize and explore the 
 gene-expression patterns and their subclusters.

vignette: >
  %\VignetteIndexEntry{ctsGE Package}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8} 
 
output: 
 BiocStyle::html_document:
    toc: true
    
---


```{r, echo = FALSE, message = FALSE}
library(ctsGE)
library(pander)
library(rmarkdown)
```

# Installing ctsGE


```{r,eval=FALSE,warning=FALSE,message=FALSE}
if (!requireNamespace("BiocManager", quietly=TRUE))
    install.packages("BiocManager")
BiocManager::install("ctsGE")
```

# Workflow of clustering with ctsGE
1. Build the expression matrix from expression data    

2. Define an expression index (i.e., a sequence of 1,-1, and 0) for each gene  

3. Cluster the gene set with the same index, applying **K-means**: 
$$kmeans(genesInIndex,k)$$  

4. Graphic visualization of expression patterns

5. Interactive visualization and exploration of gene-expression data

# Building the expression matrix  
As input, the **ctsGE** package expects a normalized expression table, where 
rows are genes and columns are samples. This can consist of count data as 
obtained, e. g., from RNA-Seq or other high-throughput sequencing experiment or
microarray experiment. Example data from the [Gene Expression Omnibus (GEO)]
(http://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE2077) are used here
illustrate ctsGE's potential. The data are the expression profile of 
*Cryptosporidium parvum*-infected human ileocecal adenocarcinoma cells 
(HCT-8),^[Deng M, Lancto CA, Abrahamsen MS (2004). 
Cryptosporidium parvum regulation of human epithelial cell gene expression. 
Int. J. Parasitol. 34, 73–82.]) comprised of 12,625 genes over 18 samples
(three replicates of six developmental stages in human cancer). For tutorial
purposes and for simplification, only one replicate out of three is used,
for six overall time points.

## Loading data from ncbi GEO 

Load the files and make a list of the `matrix`:

When reading the normalized expression values the function check whether there 
are rows that their median absolute deviation (MAD) value equal to zero and 
remove these rows. This step is important in order to continue to the next step
of indexing the data.
```{r,eval=FALSE,message=FALSE, warning=FALSE}
library(GEOquery)

gse2077 <- getGEO('GSE2077')
gseAssays <- Biobase::assayData(gse2077)
gseExprs <- Biobase::assayDataElement(gseAssays[[1]][,c(1:6)],'exprs')

# list of the time series tables use only 6 samples
gseList <- lapply(1:6,function(x){data.frame(Genes = rownames(gseExprs),Value = gseExprs[,x])}) 
names(gseList) <- colnames(gseExprs)
```

Build the expression matrix from the list of matrices:
```{r,eval=FALSE,message=FALSE,warning=FALSE }
rts <- readTSGE(gseList,labels = c("0h","6h","12h","24h","48h","72h")) 
```

## Loading data from files
Here is an example of how to load the time-series files from your directory; 
data were imported from the **ctsGE** package:
```{r, message=FALSE,warning=FALSE}
data_dir <- system.file("extdata", package = "ctsGE")
files <- dir(path=data_dir,pattern = "\\.xls$")
```

**Building from a directory:**
```{r,message=FALSE,warning=FALSE}

rts <- readTSGE(files, path = data_dir, labels = c("0h","6h","12h","24h","48h","72h") )
``` 

**ctsGE Object summary:**
```{r,message=FALSE,warning=FALSE }
names(rts)
rts$timePoints
head(rts$samples)
head(rts$tags)
```

` head(rts$tsTable)`

```{r, echo=FALSE,results='asis'}

panderOptions("table.style","rmarkdown")
pander(head(rts$tsTable))

```

### Adding genes annotation to time series table
Please use the `desc` option in the function: `readTSGE` in order to add genes 
annotation to the time series table.

# Defining the expression index  
1. First, the expression matrix is standardized. The function default 
standardizing method is a median-based scaling; alternatively, a mean-based 
scaling can be used. The new scaled values represent the distance of each gene 
at a certain time point from its center, median or mean, in median absolute 
deviation (MAD) units or standard deviation (SD) units, respectively. 

2. Next, the standardized values are converted to index values that indicate 
whether gene expression is above, below or within the limits around the center 
of the time series, i.e., **1 / -1 / 0**, respectively. The function defines a 
parameter cutoff (see Section [4.1](#checking-the-optimal-cutoff)) that 
determines the limits around the gene-expression center.Then the function 
calculates the index value at each time point according to:

    + **0:** standardized value is within the limits (+/- cutoff)
    + **1:** standardized value exceeds the upper limit (+ cutoff)
    + **-1:** standardized value exceeds the lower limit (- cutoff)

3. The ` +/- cutoff` parameter defines a reference range to which the data are 
compared. When the range is too big, more time-series points will fall into
it and will get an index value of 0, and this may be misleading. Too small range
can result in too many index groups that will be too sensitive to small
fluctuations in the time-series index. The function chooses 
the optimal cutoff (see Section [4.1](#checking-the-optimal-cutoff)) after 
testing different cutoff values from 0.5 to 0.7 in increments of 0.05. 

##Checking the optimal cutoff
The function `PreparingTheIndexes` generates an expression index 
(i.e., a sequence of 1,-1, and 0) that represents the expression pattern along
time points for each gene. Setting different limits for the center era 
(with the parameter cutoff) will change the index for each gene-expression 
profile and consequently, the number of genes in each index group. Following the
idea that instead of filtering “irrelevant” genes to reduce the noise, the 
clustering will be performed on small gene groups, one would like to choose a 
cutoff value, that will minimize the number of genes in each group, i.e., 
generate index groups of equal size. The test for equality is performed by 
calculating the chi-squared value from a comparison between the number of genes 
in the index groups and the null hypothesis that all index groups are equal. 
The test is performed for all the cutoffs in the range and the cutoff that gives
the minimal chi-squared value is the most likely to generate equal index groups.
The range of the cutoff values is given by `min_cutoff` and `max_cutoff` 
arguments. However, by setting the same value to min and max parameters one can 
define a cutoff regardless of what was suggested by the function.

```{r,message=FALSE,warning=FALSE}
prts <- PreparingTheIndexes(x = rts, min_cutoff=0.5, max_cutoff=0.7, mad.scale = TRUE)
```
**`cutoff = 0.55` is the optimal cutoff with the lowest chi-squared value**

```{r,message=FALSE,warning=FALSE}
prts$cutoff
```

### Index overview after 
To get an idea of how the data look, and to determine the nature of the indexes
formed from certain cutoff value, the number of zero values for each index 
is counted. In this tutorial example, the index can have no zeros, 
one zero or up to six zeros; overall, the indexes and genes are divided into 
seven groups. Indexes for which most of the time points present a zero value 
(in this example, three or more time points) are expected to show a pattern in 
which gene-expression does not change much along the time points. Indexes with 
less zeros to no zeros (two or less in the example) will show genes with up- or 
downregulated expression at each time point.

    With cutoff = 
    0.55, most of the genes were assigned to indexes with three or two zeros, 
    indicating a variety of expression patterns.

```{r,message=FALSE,warning=FALSE,echo=FALSE}
library(dplyr)

count_zero <- 
  function(x){
    sum(strsplit(x,"")[[1]]==0)}


tbl <- prts$index %>%
    # counting genes at each index
    group_by(index)%>% summarise(size=length(index)) %>% 
    # counting the number of zeros at each index
    group_by(index)%>% mutate(nzero=count_zero(as.character(index))) %>% 
    # groups genes by the number of zeros and sum them
    group_by(nzero) %>% summarise(genes=round(sum(size)/12625,1)) 

tmp = which(0:6%in%tbl$nzero==0)-1
tmp_df = data.frame(nzero=tmp,genes=rep(0,length(tmp)))
tbl <- bind_rows(tbl,tmp_df) %>% arrange(nzero)
labs <- seq(0,max(tbl$genes), by = 0.2)
barplot(tbl$genes, 
        main = paste("Number of zeros in indexes with cutoff =",prts$cutoff), 
        names.arg = tbl$nzero,axes = FALSE, xlab="Number of Zeros")
axis(side = 2, at = labs, labels = paste0(labs * 100, "%"))
```

##Preparing the indexes for the data  
A **`cutoff = 0.55`** was chosen
```{r, message=FALSE,warning=FALSE}
prts <- PreparingTheIndexes(x = rts, mad.scale = TRUE) 
names(prts)
```
 

**Gene expression after standardization: **
` head(prts$scaled) ` 

```{r,message=FALSE,echo=FALSE,warning=FALSE,results="asis"}

panderOptions("table.style","simple")
pander(head(prts$scaled)) 
```


**Gene expression indexing with `cutoff = 0.55`:**
` head(prts$index) `
```{r,message=FALSE,echo=FALSE,warning=FALSE,results="asis"}

panderOptions("table.style","simple")
pander(head(prts$index)) 
```

## Clustering each index with K-means  
The clustering is done with K-means. To choose an optimal k for K-means
clustering, the Elbow method was applied^[Thorndike RL (1953). Who Belongs in
the Family? Psychometrika, 18(4), 267–276.], this method looks at the percentage
of variance explained as a function of the number of clusters: the chosen number
of clusters should be such that adding another cluster does not give much better 
modeling of the data. First, the ratio of the within-cluster sum of squares 
(WSS) to the total sum of squares (TSS) is computed for different values of 
k (i.e., 1, 2, 3 ...). The WSS, also known as sum of squared error (SSE), 
decreases as k gets larger. The Elbow method chooses the k at which the SSE 
decreases abruptly. This happens when the computed value of the WSS-to-TSS ratio 
first drops from 0.2. 

$\frac{WSS}{TSS} < 0.2$
  
```{r,message=FALSE, warning=FALSE,eval=FALSE}
ClustIndexes <- ClustIndexes(prts, scaling = TRUE)
names(ClustIndexes)
# table of the index and the recommended k that were found by the function 
head(ClustIndexes$optimalK)

# Table of clusters index for each gene
head(ClustIndexes$ClusteredIdxTable) 
```

Running `kmeans` and calculating the optimal k for each one of the indexes in 
the data could take a long time. To shorten the procedure the user can skip this 
step altogether and directly view a specific index and its clusters by running either 
the `PlotIndexesClust()` or the `ctsGEShinyApp()` function

# Graphic visualization of an index  
The `PlotIndexesClust()` function generates graphs and tables of a specific 
index and its clusters. The user decides whether to supply the k or let the 
function calculate the k for the selected index. 

```{r,message=FALSE,warning=FALSE}
indexPlot <- PlotIndexesClust(prts,idx = "1100-1-1",scaling = TRUE)
names(indexPlot)
```

**Genes in '1100-1-1' index and their clusters **
**(k was chosen by the function):**  
Number of clusters (k) for '1100-1-1' is: **`length(indexPlot$graphs)`**
```{r,message=FALSE,warning=FALSE,echo=FALSE}
length(indexPlot$graphs)
```

**Table of genes in '1100-1-1' index, seperated to clusters:**
`head(indexPlot[[1]])`

```{r,message=FALSE,warning=FALSE,results="asis",echo=FALSE}

panderOptions("table.style","rmarkdown")
pander(head(indexPlot[[1]]))
```

For this example, the index `1100-1-1` is used. by Looking at this index, it can
be assumed that the expression of the genes belonging to it was downregulated 
along the time points. Since the index only states whether gene expression 
is upregulated (1), downregulated (-1) or stays the same (0), gene subsets 
of the same profile will usually show more than one expression pattern. 
K-means helps distinguish these patterns from one another.  

**Line graphs of the genes' expression patterns in index '1100-1-1' separated** 
**into clusters:**

```{r,message=FALSE,warning=FALSE}
indexPlot$graphs
```


##Export gene expression table to file
Genes expression data in index '1100-1-1' separated into clsuter can be
exported with R function `write.table`:
```
write.table(indexPlot[[1]], file, sep = "\t")
```

# GUI for interactive exploration of gene-expression data

The `ctsGEShinyApp` function the ctsGE object,and opens an html page as a GUI.
On the web page, the user chooses the profile to visualize and the number of 
clusters (k parameter for K-means) to show. The line graph of the profile 
separated into the clusters will show in the main panel, and a list of the genes
and their expressions will also be available. The tables and figures can be 
downloaded.

*\*Note that running the Shiny GUI block the current session while running.*

## Screenshots of the GUI

```{r,message=FALSE,warning=FALSE,eval=FALSE}
library(shiny)
library(DT)
ctsGEShinyApp(rts)
```

###[The graph tab](http://vitisexonusage.byethost9.com/ctsGEShinyApp_tab1.png)

### [The table tab](http://vitisexonusage.byethost9.com/ctsGEShinyApp_tab2.png)

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ctsGE.R
\name{readTSGE}
\alias{readTSGE}
\title{Read and merge a set of files containing gene expression data}
\usage{
readTSGE(files, path = NULL, columns = c(1, 2), labels = NULL,
  desc = NULL, ...)
}
\arguments{
\item{files}{character vector of filenames,
or alternative a named list of tables for each time point.}

\item{path}{character string giving the directory containing the files.
The default is the current working directory.}

\item{columns}{numeric vector stating which two columns contain the tag
names and counts, respectively}

\item{labels}{character vector giving short names to associate with the
libraries.}

\item{desc}{character vector with genes description (annotation),default to
NULL
Defaults to the file names.}

\item{...}{other are passed to read.delim}
}
\value{
A list with four objects:
 \enumerate{\item expression matrix
            \item samples names
            \item tags - genes name
            \item timePoints - number of time points}
}
\description{
Reads and merges a set of text files containing normalized gene expression
data
}
\details{
As input, the ctsGE package expects normalized expression table,
where rows are genes and columns are samples
Each file is assumed to contained digital gene expression data for one
sample (or library), with transcript or gene identifiers in the first
column and expression values in the second column.
Transcript identifiers are assumed to be unique and not repeated in any one
file.
By default, the files are assumed to be tab-delimited and to contain column
   headings.
The function forms the union of all transcripts and creates one big table
with zeros where necessary.
When reading the normalized expression values the function check whether
there are rows that their median absolute deviation (MAD) value equal to zero
and remove these rows. This step is important in order to continue to the
next step of indexing the data.
The function will output a message of how many genes were remove.
}
\examples{
## Read all .txt files from current working directory
data_dir <- system.file("extdata", package = "ctsGE")
files <- dir(path=data_dir,pattern = "\\\\.xls$")

# reading only 2000 genes
rts <- readTSGE(files, path = data_dir,
 labels = c("0h","6h","12h","24h","48h","72h"), skip = 10625 )
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PreparingTheIndexes.R
\name{PreparingTheIndexes}
\alias{PreparingTheIndexes}
\title{Define an expression index for each gene}
\usage{

PreparingTheIndexes(x, min_cutoff = 0.5, max_cutoff = 0.7,
  mad.scale = TRUE)

}
\arguments{
\item{x}{list of an expression data that made by readTSGE}


\item{min_cutoff}{A numeric the lower limit range to calculate the optimal
cutoff for the data, default to 0.5  \emph{See Details}.}

\item{max_cutoff}{A numeric the upper limit range to calculate the optimal
cutoff for the data, default to 0.7  \emph{See Details}.}


\item{mad.scale}{A boolean defaulting to TRUE as to what method of
scaling to use.
Default median-base scaling. FALSE, mean-base scaling.}
}
\value{
list object is returned as output with the relative
standarization table in \emph{object$scaled},
and the indexes table in \emph{object$index}
}
\description{
Reads the table of genes expression and return an expression index for each
gene.
}
\details{
1. First, the expression matrix is standardized. The function
default standardizing method is a median-based scaling; alternatively, a
mean-based scaling can be used. The new scaled values represent the distance
of each gene at a certain time point from its center, median or mean,
in median absolute deviation (MAD) units or standard deviation (SD) units,
respectively.

2. The function compute the cutoff value following the idea that the
clustering will be performed on small gene groups, an optimal cutoff value
will be one that will minimize the number of genes in each group,
i.e., generate index groups of equal size. The chi-squared values will be
generate for each cutoff value (from min_cutoff to max_cutoff parameter
in increments of 0.05) the cutoff that generate the lowest chi-squared is
chosen.

3. Next, the standardized values are converted to index values that indicate
whether gene expression is above, below or within the limits around the
center of the time series, i.e., **1 / -1 / 0**, respectively. The cutoff
parameter determines the limits around the gene-expression center.
Then the function calculates the index value at each
time point according to:

\enumerate{
     \item \bold{0:}  standardized value is within the limits (+/- cutoff)
     \item \bold{1:}  standardized value exceeds the upper limit (+ cutoff)
     \item \bold{-1:} standardized value exceeds the lower limit (- cutoff)}
}
\examples{
data_dir <- system.file("extdata", package = "ctsGE")
files <- dir(path=data_dir,pattern = "\\\\.xls$")
rts <- readTSGE(files, path = data_dir,
labels = c("0h","6h","12h","24h","48h","72h"), skip = 10625 )
prts <- PreparingTheIndexes(rts)
prts$cutoff # the optimal cutoff

}
\seealso{
\code{\link{scale}} \code{\link{index}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ctsGEShinyApp.R
\name{ctsGEShinyApp}
\alias{ctsGEShinyApp}
\title{GUI for interactive exploration of gene expression data.}
\usage{

ctsGEShinyApp(rts, min_cutoff = 0.5, max_cutoff = 0.7, mad.scale = TRUE,
  title = NULL)

}
\arguments{
\item{rts}{list of an expression data that made by readTSGE}

\item{min_cutoff}{A numeric the lower limit range to calculate the optimal
cutoff for the data, default to 0.5
\emph{See \code{\link{PreparingTheIndexes}}}.}

\item{max_cutoff}{A numeric the upper limit range to calculate the optimal
cutoff for the data, default to 0.7
\emph{See \code{\link{PreparingTheIndexes}}}.}


\item{mad.scale}{A boolean defaulting to TRUE as to what method of scaling
to use.
 Default median-base scaling. FALSE, mean-base scaling}

\item{title}{Character, the title at the header panel. default to NULL.}
}
\value{
Creates a shiny application and opens a shinyapp.io web page
}
\description{
Produce and launch Shiny app for interactive exploration of gene expression
 data.
For more information about shiny apps \cite{http://shiny.rstudio.com/}
}
\details{
The `ctsGEShinyApp` function takes the ctsGE object
and opens an html page as a GUI. On the web page, the user
chooses the profile to visualize and the number of clusters (k parameter for
 K-means) to show. The line graph of the profile separated into the clusters
 will show in the main panel, and a list of the genes and their expressions
 will also be available. The tables and figures can be downloaded.
}
\examples{

\dontrun{
data_dir <- system.file("extdata", package = "ctsGE")
files <- dir(path=data_dir,pattern = "\\\\.xls$")
rts <- readTSGE(files, path = data_dir,
labels = c("0h","6h","12h","24h","48h","72h") )
ctsGEShinyApp(rts)}


}
\seealso{
shiny::ShinyApp
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PreparingTheIndexes.R
\name{index}
\alias{index}
\title{Indexing function}
\usage{
index(x, cutoff)
}
\arguments{
\item{x}{A numeric}

\item{cutoff}{A numeric, dermine the threshold for indexing}
}
\value{
Gene expression index
}
\description{
Takes a numeric vector and return an expression index
(i.e., a sequence of 1,-1, and 0)
}
\details{
The function defines limits around the center (median or mean),
+/- cutoff value in median absolute deviation (MAD) or standard deviation
(SD) units respectively.The user defines a parameter cutoff that determines
the limits around the gene-expression center. Then the function calculates
the index value at each time point according to:

\enumerate{
     \item \bold{0:}  standardized value is within the limits (+/- cutoff)
     \item \bold{1:}  standardized value exceeds the upper limit (+ cutoff)
     \item \bold{-1:} standardized value exceeds the lower limit (- cutoff)}
}
\examples{
rawCounts <-
c(103.5, 75.1, 97.3, 27.12, 34.83, 35.53, 40.59, 30.84, 16.39, 29.29)

(sCounts <- scale(rawCounts)[,1])# standardized mean-base scaling

cutoff <- seq(0.2,2,0.1)    # different cutoff produce different indexes

for(i in cutoff){print(index(sCounts,i))}
}
\seealso{
\code{\link{PreparingTheIndexes}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ClustIndexes.R
\name{ClustIndexes}
\alias{ClustIndexes}
\title{Clustering the indexes applying K-means}
\usage{
ClustIndexes(x, scaling = TRUE)
}
\arguments{
\item{x}{list of expression data and their indexes after running
\code{\link{PreparingTheIndexes}}}

\item{scaling}{Boolean parameter, does the data should be standardized
before clustered. Default = TRUE}
}
\value{
list object is returned as output,
with the relative culstered indexes table in
 \emph{object$ClusteredIdxTable},
and the number of clusters for each index in \emph{object$optimalK}
}
\description{
Clustering each index, that was predifined by
\code{\link{PreparingTheIndexes}}, with \code{\link{kmeans}}.
}
\details{
The clustering is done with K-means. To choose an optimal k for
 K-means clustering, the Elbow method was applied, this method looks at the
 percentage of variance explained as a function of the number of clusters: the
 chosen number of clusters should be such that adding another cluster does
 not give much better modeling of the data. First, the ratio of the
 within-cluster sum of squares (WSS) to the total sum of squares (TSS) is
 computed for different values of k (i.e., 1, 2, 3 ...). The WSS, also known
 as sum of squared error (SSE), decreases as k gets larger. The Elbow method
 chooses the k at which the SSE decreases abruptly. This happens when the
 computed value of the WSS-to-TSS ratio first drops from 0.2.

 Running \code{\link{kmeans}} and calculating the optimal k for each one of
 the indexes in the data could take a long time. To shorten the procedure the
 user can skip this step altogether and directly view a specific index and
 its clusters by running either the \code{\link{PlotIndexesClust}} or the
 \code{\link{ctsGEShinyApp}} function.

 By default data is standardize before clustering,for clustering
 the raw counts set the \command{\strong{scaling}} parameter to FALSE.
}
\examples{

data_dir <- system.file("extdata", package = "ctsGE")
files <- dir(path=data_dir,pattern = "\\\\.xls$")
rts <- readTSGE(files, path = data_dir,
labels = c("0h","6h","12h","24h","48h","72h"), skip = 10625 )
prts <- PreparingTheIndexes(rts)

tsCI <- ClustIndexes(prts)

head(tsCI$ClusteredIdxTable) #the table with the clusterd indexes
head(tsCI$optimalK) #the table with the number of clusters for each index

}
\seealso{
\code{\link{kmeans}}, \code{\link{PlotIndexesClust}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PlotIndexesClust.R
\name{PlotIndexesClust}
\alias{PlotIndexesClust}
\title{Graphic visualization of an index}
\usage{
PlotIndexesClust(x, idx, k = NULL, scaling = TRUE)
}
\arguments{
\item{x}{list of expression data and their indexes after running
\code{\link{PreparingTheIndexes}}}

\item{idx}{A character, the index to plot
(e.g., for 8 time points "11100-1-1-1")}

\item{k}{A numeric, number of clusters. If not given the function will
calculate what is the optimal k for the index.}

\item{scaling}{A boolean, default to TRUE,
does the data should be standardized before clustered with K-means.}
}
\value{
A list with two objects:
 \enumerate{\item Table of of a specific index and its clusters
            \item Gene expression pattern graphs
            for each one of the clusters}
}
\description{
The function generates graphs and tables of a specific index and its
clusters. The user decides whether to supply the k or let the function
calculate the k for the selected index
}
\examples{

data_dir <- system.file("extdata", package = "ctsGE")
files <- dir(path=data_dir,pattern = "\\\\.xls$")
rts <- readTSGE(files, path = data_dir,
labels = c("0h","6h","12h","24h","48h","72h"), skip = 10625 )
prts <- PreparingTheIndexes(rts)
pp <- PlotIndexesClust(prts,idx="00101-1")
pp$graphs # plots the line graphs

}
\seealso{
\code{\link{ggplot}},  \code{\link{kmeans}},
 \code{\link{ClustIndexes}}
}
